@@ -1,8 +1,7 @@
-# $Id: GenBank.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::DB::GenBank
 #
-# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+# Please direct questions and support issues to <bioperl-l@bioperl.org>
 #
 # Cared for by Aaron Mackey <amackey@virginia.edu>
 #
@@ -11,7 +10,7 @@
 # You may distribute this module under the same terms as perl itself
 #
 # POD documentation - main docs before the code
-# 
+#
 # Added LWP support - Jason Stajich 2000-11-6
 # completely reworked by Jason Stajich 2000-12-8
 # to use WebDBSeqI
@@ -33,7 +32,7 @@ Bio::DB::GenBank - Database object interface to GenBank
     use Bio::DB::GenBank;
     $gb = Bio::DB::GenBank->new();
 
-    $seq = $gb->get_Seq_by_id('MUSIGHBA1'); # Unique ID
+    $seq = $gb->get_Seq_by_id('J00522'); # Unique ID, *not always the LOCUS ID*
 
     # or ...
 
@@ -57,11 +56,11 @@ Bio::DB::GenBank - Database object interface to GenBank
 
     # also don't want features, just sequence so let's save bandwith
     # and request Fasta sequence
-    $gb = Bio::DB::GenBank->new(-retrievaltype => 'tempfile' , 
+    $gb = Bio::DB::GenBank->new(-retrievaltype => 'tempfile' ,
 			                      -format => 'Fasta');
     my $seqio = $gb->get_Stream_by_acc(['AC013798', 'AC021953'] );
     while( my $clone =  $seqio->next_seq ) {
-      print "cloneid is ", $clone->display_id, " ", 
+      print "cloneid is ", $clone->display_id, " ",
              $clone->accession_number, "\n";
     }
     # note that get_Stream_by_version is not implemented
@@ -131,15 +130,15 @@ of the Bioperl mailing lists. Your participation is much appreciated.
   bioperl-l@bioperl.org                  - General discussion
   http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
-=head2 Support 
+=head2 Support
 
 Please direct usage questions or support issues to the mailing list:
 
 I<bioperl-l@bioperl.org>
 
-rather than to the module maintainer directly. Many experienced and 
-reponsive experts will be able look at the problem and quickly 
-address it. Please include a thorough description of the problem 
+rather than to the module maintainer directly. Many experienced and
+reponsive experts will be able look at the problem and quickly
+address it. Please include a thorough description of the problem
 with code and data examples if at all possible.
 
 =head2 Reporting Bugs
@@ -148,7 +147,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.  Bug reports can be submitted via the
 web:
 
-  http://bugzilla.open-bio.org/
+  https://github.com/bioperl/bioperl-live/issues
 
 =head1 AUTHOR - Aaron Mackey, Jason Stajich
 
@@ -170,11 +169,11 @@ use strict;
 use vars qw(%PARAMSTRING $DEFAULTFORMAT $DEFAULTMODE);
 
 use base qw(Bio::DB::NCBIHelper);
-BEGIN {    
+BEGIN {
     $DEFAULTMODE   = 'single';
     $DEFAULTFORMAT = 'gbwithparts';
     %PARAMSTRING = (
-			 'batch' => { 'db'     => 'nucleotide',
+		    'batch' => { 'db'     => 'nucleotide',
 				  'usehistory' => 'n',
 				  'tool'   => 'bioperl'},
 		     'query' => { 'usehistory' => 'y',
@@ -192,7 +191,7 @@ BEGIN {
 				   'usehistory' => 'n',
 				   'tool'   => 'bioperl',
 				   'retmode' => 'text'},
-			 'webenv' => {    
+		      'webenv' => {
 				  'query_key'  => 'querykey',
 				  'WebEnv'  => 'cookie',
 				  'db'     => 'nucleotide',
@@ -211,7 +210,7 @@ BEGIN {
  Title   : new
  Usage   : $gb = Bio::DB::GenBank->new(@options)
  Function: Creates a new genbank handle
- Returns : New genbank handle
+ Returns : a new Bio::DB::Genbank object
  Args    : -delay   number of seconds to delay between fetches (3s)
 
 NOTE:  There are other options that are used internally.  By NCBI policy, this
@@ -257,7 +256,12 @@ sub get_params {
   Function: Gets a Seq object by accession numbers
   Returns : a Bio::Seq object
   Args    : the accession number as a string
-  Note    : For GenBank, this just calls the same code for get_Seq_by_id()
+  Note    : For GenBank, this just calls the same code for get_Seq_by_id().
+            Caveat: this normally works, but in rare cases simply passing the
+            accession can lead to odd results, possibly due to unsynchronized
+            NCBI ID servers. Using get_Seq_by_version() is slightly better, but
+            using the unique identifier (GI) and get_Seq_by_id is the most
+            consistent
   Throws  : "id does not exist" exception
 
 =head2 get_Seq_by_gi
@@ -276,6 +280,8 @@ sub get_params {
  Function: Gets a Bio::Seq object by sequence version
  Returns : A Bio::Seq object
  Args    : accession.version (as a string)
+ Note    : Caveat: this normally works, but using the unique identifier (GI) and
+           get_Seq_by_id is the most consistent
  Throws  : "acc.version does not exist" exception
 
 =head1 Routines implemented by Bio::DB::NCBIHelper
@@ -346,7 +352,7 @@ instead.
  Title   : get_request
  Usage   : my $url = $self->get_request
  Function: HTTP::Request
- Returns : 
+ Returns :
  Args    : %qualifiers = a hash of qualifiers (ids, format, etc)
 
 =cut
