@@ -4,6 +4,8 @@
 #
 # BioPerl module for Bio::DB::Query::LANLQuery
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Mark A. Jensen <maj@fortinbras.us>
 #
 # Copyright Mark A. Jensen
@@ -106,18 +108,17 @@ An attempt was made to make the query field names natural and easy to
 remember. Aliases are specified in an XML file (C<lanl-schema.xml>) that is part
 of the distribution. Custom field aliases can be set up by modifying this file.
 
-An HTML cheatsheet with valid field names, aliases, and match data can
-be generated from the XML by using
-C<hiv_object-E<gt>help('help.html')>. A query can also be validated
-locally before it is unleashed on the server; see below.
+An HTML cheatsheet with valid field names, aliases, and match data can be
+generated from the XML by using C<hiv_object-E<gt>help('help.html')>. A query
+can also be validated locally before it is unleashed on the server; see below.
 
 =head2 Annotations
 
 LANL DB annotations have been organized into a number of natural
-groupings, tagged C<Geo>, C<Patient>, C<Virus>, and <StdMap>.  After a
+groupings, tagged C<Geo>, C<Patient>, C<Virus>, and C<StdMap>.  After a
 successful query, each id is associated with a tree of
 L<Bio::Annotation::SimpleValue> objects. These can be accessed with
-methods C<get_value()> and C<put_value()> described in APPENDIX.
+methods C<get_value> and C<put_value> described in APPENDIX.
 
 =head2 Delayed/partial query runs
 
@@ -161,13 +162,24 @@ the Bioperl mailing list.  Your participation is much appreciated.
   bioperl-l@bioperl.org                  - General discussion
   http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
+
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
 of the bugs and their resolution. Bug reports can be submitted via
 the web:
 
-  http://bugzilla.open-bio.org/
+  https://github.com/bioperl/bioperl-live/issues
 
 =head1 AUTHOR - Mark A. Jensen
 
@@ -175,6 +187,8 @@ Email maj@fortinbras.us
 
 =head1 CONTRIBUTORS
 
+Mark A. Jensen
+
 =head1 APPENDIX
 
 The rest of the documentation details each of the object methods.
@@ -387,16 +401,16 @@ sub help{
    my ($self, $fname) = @_;
    my (@ret, @tok);
    my $schema = $self->_schema;
-   my $h = new CGI;
+   my $h = CGI->new();
 
    my (@tbls, @flds, @als, @opts, $fh);
    if ($fname) {
-       open ($fh, ">", $fname) or $self->throw(-class=>'Bio::Root::IOException',
-                                               -text=>"Error opening help html file $fname for writing",
-                                               -value=>$!);
+       open $fh, '>', $fname or $self->throw(-class => 'Bio::Root::IOException',
+                                             -text  => "Error opening help html file $fname for writing",
+                                             -value => $!);
    }
    else {
-       open($fh, ">&1");
+       open $fh, ">&1";
    }
    @tbls = $schema->tables;
    @tbls = ('COMMAND', grep !/COMMAND/,@tbls);
@@ -414,44 +428,50 @@ sub help{
        @flds = grep /^$tbl/, $schema->fields;
        @flds = grep !/_id/, @flds;
        print $fh (
-	   $h->start_Tr({-style=>"background-color: lightblue;"}), 
-	   $h->td([$h->a({-id=>$tbl},$tbl), $h->span({-style=>"font-style:italic"},"fields"), $h->span({-style=>"font-style:italic"}, "aliases")]),
-	   $h->end_Tr
-	   );
+           $h->start_Tr({-style=>"background-color: lightblue;"}), 
+           $h->td([$h->a({-id=>$tbl},$tbl), $h->span({-style=>"font-style:italic"},"fields"), $h->span({-style=>"font-style:italic"}, "aliases")]),
+           $h->end_Tr
+       );
        foreach my $fld (@flds) {
-	   @als = reverse $schema->aliases($fld);
-	   print $fh (
-	       $h->Tr( $h->td( ["", $h->a({-href=>"#opt$fld"}, shift @als), $h->code(join(',',@als))] ))
-	       );
-       my @tmp = grep {$_} $schema->options($fld);
-	   @tmp = sort {(($a =~ /^[0-9]+$/) && $b =~ /^[0-9]+$/) ? $a<=>$b : $a cmp $b} @tmp;
-	   if (grep /Any/,@tmp) {
-	       @tmp = grep !/Any/, @tmp;
-	       unshift @tmp, 'Any';
-	   }
-       #print STDERR join(', ',@tmp)."\n";
-	   push @opts, $h->div(
-	       {-style=>"font-family:sans-serif;font-size:small"},
-	       $h->hr,
-	       $h->a(
-		   {-id=>"opt$fld"},
-		   "<i>Valid options for</i> <b>$fld</b>: "),
-	       $h->blockquote(
-		   @tmp ? $h->code(join(", ", @tmp)) : $h->i("free text")
-	       ),
-	       $h->span(
-		   "<i>Other aliases</i>: "),
-	       $h->blockquote(
-		   @als ? $h->code(join(",",@als)) : "<i>none</i>"
-	       ),
-	       " ", 
-	       $h->table( $h->Tr( 
-			      $h->td([
-				  $h->a({-href=>"#$tbl"}, $h->small('BACK')), 
-				  $h->a({-href=>"#TOP"}, $h->small('TOP'))
-				     ]) ) )
-	       );
-	   
+           @als = reverse $schema->aliases($fld);
+           print $fh (
+               # note that aliases can sometimes be empty
+               $h->Tr( $h->td( ["", $h->a({-href=>"#opt$fld"}, shift @als || '???'), $h->code(join(',',@als))] ))
+           );
+           my @tmp = grep {$_} $schema->options($fld);
+           @tmp = sort {(($a =~ /^[0-9]+$/) && $b =~ /^[0-9]+$/) ? $a<=>$b : $a cmp $b} @tmp;
+           if (grep /Any/,@tmp) {
+               @tmp = grep !/Any/, @tmp;
+               unshift @tmp, 'Any';
+           }
+           #print STDERR join(', ',@tmp)."\n";
+           push @opts, $h->div(
+               {-style=>"font-family:sans-serif;font-size:small"},
+               $h->hr,
+               $h->a(
+                   {-id=>"opt$fld"},
+                   "<i>Valid options for</i> <b>$fld</b>: "
+               ),
+               $h->blockquote(
+                   @tmp ? $h->code(join(", ", @tmp)) : $h->i("free text")
+               ),
+               $h->span(
+                   "<i>Other aliases</i>: "
+               ),
+               $h->blockquote(
+                   @als ? $h->code(join(",",@als)) : "<i>none</i>"
+               ),
+               " ", 
+               $h->table(
+                   $h->Tr(
+                       $h->td([
+                           $h->a({-href=>"#$tbl"}, $h->small('BACK')), 
+                           $h->a({-href=>"#TOP"}, $h->small('TOP'))
+                       ])
+                   )
+               )
+           );
+   
        }
    }
    print $fh $h->end_table;
@@ -510,7 +530,8 @@ sub get_annotations_by_id {
 sub add_annotations_for_id{
     my $self = shift;
     my ($id, $ac) = @_;
-    $ac = new Bio::Annotation::Collection unless defined $ac;
+    $id = "" unless defined $id; # avoid warnings
+    $ac = Bio::Annotation::Collection->new() unless defined $ac;
     $self->throw(-class=>'Bio::Root::BadParameter'
 		 -text=>'Bio::Annotation::Collection required at arg 2',
 		 -value=>"") unless ref($ac) eq 'Bio::Annotation::Collection';
@@ -601,6 +622,18 @@ sub remove_annotations {
 
 =cut
 
+=head2 get_keys
+
+ Title   : get_keys
+ Usage   : $ac->get_keys($tagname_level_1, $tagname_level_2,...)
+ Function: Get an array of tagnames underneath the named tag nodes
+ Example : # prints the values of the members of Category 1...
+           print map { $ac->get_value($_) } $ac->get_keys('Category 1') ;
+ Returns : array of tagnames or empty list if the arguments represent a leaf
+ Args    : [array of] tagname[s]
+
+=cut
+
 =head1 GenBank accession manipulation methods
 
 =head2 get_accessions
@@ -625,7 +658,7 @@ sub get_accessions{
     }
     my @ac = $self->get_annotations_by_ids($self->ids);
     foreach (@ac) {
-	push @ret, $_->get_value('accession');
+	push @ret, $_->get_value('Special','accession');
     };
     return @ret;
 }
@@ -653,7 +686,7 @@ sub get_accessions_by_ids {
     }
     my @ac = $self->get_annotations_by_ids(@ids);
     foreach (@ac) {
-	push @ret, $_->get_value('accession');
+	push @ret, $_->get_value('Special', 'accession');
     };
     return wantarray ? @ret : $ret[0];
 }
@@ -745,6 +778,23 @@ sub _session_id{
     return $self->{'_session_id'} = shift if @_;
     return $self->{'_session_id'};
 }
+=head2 _run_level
+
+ Title   : _run_level
+ Usage   : $obj->_run_level($newval)
+ Function: returns the level at which the query has so far been run
+ Example : 
+ Returns : value of _run_level (a scalar)
+ Args    : on set, new value (a scalar or undef, optional)
+
+=cut
+
+sub _run_level{
+    my $self = shift;
+
+    return $self->{'_RUN_LEVEL'} = shift if @_;
+    return $self->{'_RUN_LEVEL'};
+}
 
 =head2 _run_option
 
@@ -815,6 +865,7 @@ sub _ua_hash{
 sub add_id {
     my $self = shift;
     my $id = shift;
+    $id = "" unless defined $id; # avoid warnings
     ${$self->{'ids'}}{$id}++;
     return $id;
 }
@@ -1252,6 +1303,8 @@ sub _do_lanl_request {
     my $seqs_found_re = qr{Displaying$tags_re*(?:\s*[0-9-]*\s*)*$tags_re*of$tags_re*\s*([0-9]+)$tags_re*sequences found};
     my $no_seqs_found_re = qr{Sorry.*no sequences found};
     my $too_many_re = qr{too many records: $tags_re*([0-9]+)};
+    my $sys_error_re = qr{[Ss]ystem error};
+    my $sys_error_extract_re = qr{${tags_re}error:.*?<td[^>]+>${tags_re}(.*?)<br>};
     # find something like:
     #  <strong>tables without join:</strong><br>SequenceAccessions<br>
     my $tbl_no_join_re = qr{tables without join}i;
@@ -1287,10 +1340,12 @@ sub _do_lanl_request {
 	# squish fieldnames into hash keys
 	my %q = @query;
 	@interface = grep {defined} map {my ($tbl,$col) = /^(.*)\.(.*)$/} keys %q;
+	my $err_val = ""; # to contain informative (ha!) value if error is parsed
+
 	eval { # encapsulate communication errors here, defer biothrows...
         
         #mark the useragent should be setable from outside (so we can modify timeouts, etc)
-	    my $ua = new Bio::WebAgent($self->_ua_hash);
+	    my $ua = Bio::WebAgent->new($self->_ua_hash);
 	    my $idPing = $ua->get($self->_map_db_uri);
 	    $idPing->is_success || do {
 		$response=$idPing; 
@@ -1331,10 +1386,12 @@ sub _do_lanl_request {
 	    $response = $searchGet;
 	    for ($searchGet->content) {
 		/$no_seqs_found_re/ && do {
+		    $err_val = 0;
 		    die "No sequences found";
 		    last;
 		};
 		/$too_many_re/ && do {
+		    $err_val = $1;
 		    die "Too many records ($1): must be <10000";
 		    last;
 		};
@@ -1342,6 +1399,11 @@ sub _do_lanl_request {
 		    die "Some required tables went unjoined to query";
 		    last;
 		};
+		/$sys_error_re/ && do {
+		    /$sys_error_extract_re/;
+		    $err_val = $1;
+		    die "LANL system error";
+		};
 		/$seqs_found_re/ && do {
 		    $numseqs = $1;
 		    $count += $numseqs;
@@ -1363,7 +1425,7 @@ sub _do_lanl_request {
 	    ($@ !~ "No sequences found") && do {
 		$self->throw(-class=>'Bio::WebError::Exception',
 			     -text=>$@,
-			     -value=>"");
+			     -value=>$err_val);
 	    };
 	}
     }
@@ -1393,16 +1455,9 @@ sub _parse_lanl_response {
     my $self = shift;
     
     my ($seqGet) = (@_);
-    my (@data, @cols, %antbl, %antype,%anxlt, @ankeys );
+    my (@data, @cols, %antbl, %antype);
     my $numseq = 0;
     my ($schema, @retseqs, %rec, $ac);
-    my %specials = (
-	'country' => 'sample_country',
-	'coreceptor' => 'second_receptor',
-	'patient health' => 'health_status',
-	'year' => 'sample_year'
-	);
-    
     $schema = $self->_schema;
     
     $self->_lanl_response || 
@@ -1411,55 +1466,31 @@ sub _parse_lanl_response {
 		     -value=>"");
     foreach my $rsp (@{$self->_lanl_response}) {
 	@data = split(/\r|\n/, $rsp->content);
-	$numseq += ( shift(@data) =~ /Number.*:\s([0-9]+)/ )[0];
-	@cols = split(/\t/, shift @data);
-
+	my $l;
+	do {
+	    $l = shift @data;
+	} while ($l !~ /Number/);
+	$numseq += ( $l =~ /Number.*:\s([0-9]+)/ )[0];
+	@cols = split(/\t/, shift(@data));
 	# mappings from column headings to annotation keys
 	# squish into hash keys
 	my %q = @{ shift @{$self->_lanl_query} };
 	%antbl = $schema->ankh(keys %q);
-	foreach (values %antbl) { 
-	    #normalize
-	    my $k = $_->{ankey};
-	    $k =~ tr/ /_/;
-	    $k = lc $k;
-	    $_->{ankey} = $k; #replace with normalized version
-	    $antype{$k} = $_->{antype};
-	    push @ankeys, $k;
-	}
-	foreach (@cols) { #these are the data column headers
-	    # normalize:
-	    tr/ /_/;
-	    my $c = lc $_;
-	    ### conversion kludge for specials
-	    ### (i.e.,column headers that do not match the 
-	    ###  true field names)
-	    $c = $specials{$c} if (grep /$c/, keys %specials);
-	    ###
-	    $c =~ tr/ /_/;
-	    ### following line grep: looks for a match of the 
-	    ### column name at the end of the true field names to 
-	    ### make the translation...
-	    ### only captures the first match.
-	    my ($match_fld) = grep (/$c$/i, keys %antbl);
-	    $anxlt{$_} = $antbl{$match_fld}->{ankey} if $match_fld;
-	}
+	# get the category for each annotation
+	map { $antype{ $_->{ankey} } = $_->{antype} } values %antbl;
+	# normalize column headers
+	map { tr/ /_/; $_ = lc; } @cols;
 	foreach (@data) {
 	    @rec{@cols} = split /\t/;
-	    my $id = $rec{'SE_id'};
-	
+	    my $id = $rec{'se_id'};
 	    $self->add_id($id);
-	    $ac = new Bio::Annotation::Collection();
-
+	    $ac = Bio::Annotation::Collection->new();
 	    #create annotations
-	    # need to handle reference, comment, dblink annots
 	    foreach (@cols) {
-		#accession should be added in here as a matter of course
-		my $k = $anxlt{$_}; # annot key
-		next unless $k;
-		my $t = $antype{$k}; # annot type
+                next if $_ eq '#';
+		my $t = $antype{$_} || "Unclassified";
 		my $d = $rec{$_}; # the data
-                $ac->put_value(-KEYS=>[$t, $k], -VALUE=>$d) if $k;
+                $ac->put_value(-KEYS=>[$t, $_], -VALUE=>$d);
 	    }
 	    $self->add_annotations_for_id($id, $ac);	
 	}
@@ -1469,7 +1500,7 @@ sub _parse_lanl_response {
 }
     
 =head2 _parse_query_string
-    
+
  Title   : _parse_query_string
  Usage   : $hiv_query->_parse_query_string($str)
  Function: Parses a query string using query language emulator QRY
@@ -1513,5 +1544,3 @@ sub _sorry{
 }
 
 1;
-
- 
