@@ -35,7 +35,7 @@ via HTTP
 
   my $factory = Bio::Tools::Run::RemoteBlast->new(@params);
 
-  #change a query paramter
+  #change a query parameter
   $Bio::Tools::Run::RemoteBlast::HEADER{'ENTREZ_QUERY'} = 'Homo sapiens [ORGN]';
 
   #change a retrieval parameter
@@ -49,6 +49,9 @@ via HTTP
 
   my $str = Bio::SeqIO->new(-file=>'amino.fa' , -format => 'fasta' );
 
+  #optional: send BLAST request to a cloud service provider instead of NCBI
+  #$factory->set_url_base("http://host.my.cloud.service.provider.com/cgi-bin/blast.cgi");
+
   while (my $input = $str->next_seq()){
     #Blast a sequence against a database:
 
@@ -133,7 +136,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.  Bug reports can be submitted via the
 web:
 
-  https://redmine.open-bio.org/projects/bioperl/
+  https://github.com/bioperl/bioperl-live/issues
 
 =head1 AUTHOR 
 
@@ -161,6 +164,7 @@ use IO::String;
 use Bio::SearchIO;
 use LWP;
 use HTTP::Request::Common;
+use Bio::Root::Version;
 
 use constant {
     NOT_FINISHED => 0,
@@ -175,7 +179,7 @@ use base qw(Bio::Root::IO Exporter);
 
 our @EXPORT = qw( NOT_FINISHED ERR_QBSTATUS ERR_NOCONTENT ERR_HTTPFAIL ERR_QBNONSPEC );
 our $MODVERSION = $Bio::Root::Version::VERSION;
-our $URLBASE = 'http://www.ncbi.nlm.nih.gov/blast/Blast.cgi';
+our $URLBASE = 'http://blast.ncbi.nlm.nih.gov/Blast.cgi';
 
 # In GET/PUTPARAMS the values are regexes which validate the input.
 our %PUTPARAMS = (
@@ -645,30 +649,32 @@ sub retrieve_blast {
 =cut
 
 sub save_output {
-        my ($self, $filename) = @_;
-        if( ! defined $filename ) {
-                $self->throw("Can't save blast output.  You must specify a filename to save to.");
-        }
-        my $blastfile = $self->file;
-        #open temp file and output file, have to filter out some HTML
-        open(my $TMP, $blastfile) or $self->throw("cannot open $blastfile");
+    my ($self, $filename) = @_;
+    if( not defined $filename ) {
+        $self->throw("Can't save blast output.  You must specify a filename to save to.");
+    }
+    my $blastfile = $self->file;
+    #open temp file and output file, have to filter out some HTML
+    open my $TMP, '<', $blastfile or $self->throw("Could not read file '$blastfile': $!");
 
-        open(my $SAVEOUT, ">", $filename) or $self->throw("cannot open $filename");
-        my $seentop = 0;
-        while(<$TMP>) {
-                next if (/<pre>/);
-                if(/^(?:[T]?BLAST[NPX])\s*.+$/i ||
-           /^RPS-BLAST\s*.+$/i ||
-           /<\?xml\sversion=/ ||
-           /^#\s+(?:[T]?BLAST[NPX])\s*.+$/) {
-                        $seentop=1;
-                } 
-        next if !$seentop;
-                if( $seentop ) {
-                        print $SAVEOUT $_;
-                }
+    open my $SAVEOUT, '>', $filename or $self->throw("Could not write file '$filename': $!");
+    my $seentop = 0;
+    while (my $line = <$TMP>) {
+        next if ($line =~ /<pre>/);
+
+        if (   $line =~ /^(?:[T]?BLAST[NPX])\s*.+$/i
+            or $line =~ /^RPS-BLAST\s*.+$/i
+            or $line =~ /<\?xml\sversion=/
+            or $line =~ /^#\s+(?:[T]?BLAST[NPX])\s*.+$/
+           ) {
+            $seentop = 1;
         }
-        return 1;
+        next if !$seentop;
+        if ( $seentop ) {
+            print $SAVEOUT $line;
+        }
+    }
+    return 1;
 }
 
 sub _load_input {
@@ -710,11 +716,13 @@ sub _load_input {
 
  Title   : set_url_base
  Usage   : $self->set_url_base($url)
- Function: Method to override the default NCBI BLAST database
+ Function: Method to override the default URL to access the NCBI BLAST web service
  Returns : None
- Args    : string (database url like
+ Args    : string (URL used for remote BLAST searches)
  NOTE    : This is highly experimental; we cannot maintain support on
-           databases other than the default NCBI database at this time
+           web services other than the default NCBI BLAST web service at this
+           time. Only some URL parameters may be supported by other BLAST
+           web services.
 
 =cut
 
@@ -727,7 +735,7 @@ sub set_url_base {
 
  Title   : get_url_base
  Usage   : my $url = $self->set_url_base
- Function: Get the current URL for BLAST database searching
+ Function: Get the current URL for BLAST searching
  Returns : string (URL used for remote blast searches)
  Args    : None
 
