@@ -14,7 +14,7 @@ my $gb = new Bio::DB::GenBank;
 my $seqout = new Bio::SeqIO(-fh => \*STDOUT, -format => 'fasta');
 
 # if you want a single seq
-my $seq = $gb->get_Seq_by_id('MUSIGHBA1');
+my $seq = $gb->get_Seq_by_id('J00522');
 $seqout->write_seq($seq);
 # or by accession
 $seq = $gb->get_Seq_by_acc('AF303112');
@@ -23,7 +23,7 @@ $seqout->write_seq($seq);
 
 # feel free to pull multiple sequences...
 # if you want to get a bunch of sequences use the get_Stream_by_id/acc methods
-my $seqio = $gb->get_Stream_by_id([ qw(J00522 AF303112 2981014)]); 
+my $seqio = $gb->get_Stream_by_id([ qw(J00522 AF303112 2981014)]);
 
 while( defined ($seq = $seqio->next_seq )) {
         $seqout->write_seq($seq);
