@@ -1,4 +1,4 @@
-#!/usr/bin/perl
+#!/usr/local/bin/perl 
 
 # Example usage of a SearchIO::psiblast parser of traditional format Blast 
 # and PSI-Blast reports.
@@ -23,6 +23,7 @@
 # Author: Steve Chervitz <sac@bioperl.org>
 
 use strict;
+use lib '../../';
 use Bio::SearchIO;
 
 my $in = Bio::SearchIO->new( -format => 'psiblast',
